import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:shared_preferences/shared_preferences.dart';

typedef HttpProgressListener(int totalBytes, int receivedBytes, bool done);

class Http {
  static String baseURL;
  static String _token;

  static Future<bool> loadAccessToken() async {
    SharedPreferences pref = await SharedPreferences.getInstance();
    if (pref.containsKey("token")) {
      _token = pref.getString("token");
      return true;
    }

    return false;
  }

  static Future<bool> setAccessToken(String token) async {
    _token = token;

    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      if (await pref.setString("token", token)) {
        return true;
      }
    } catch (e) {}

    return false;
  }

  static Future<bool> removeAccessToken() async {
    _token = null;
    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      if (pref.containsKey("token")) {
        await pref.remove("token");
        return true;
      }
    } catch (e) {}

    return false;
  }

  static _processHeader({
    HttpClientRequest request,
    Map<String, dynamic> headers,
    bool useAuthorization = true,
  }) {
    if (headers != null) {
      headers.forEach((key, value) {
        request.headers.add(key, value);
      });
    }

    if (useAuthorization && _token != null) {
      request.headers.set("Authorization", _token);
    }
  }

  static _processResponse(
      {HttpClientResponse response,
      HttpProgressListener progressListener}) async {
    final int totalBytes = response.contentLength;
    int receivedBytes = 0;

    String body = await response
        .transform(
          StreamTransformer.fromHandlers(
            handleData: (data, sink) {
              sink.add(data);

              if (progressListener != null) {
                receivedBytes += data.length;
                progressListener(totalBytes, receivedBytes, false);
              }
            },
            handleDone: (sink) {
              sink.close();
              if (progressListener != null) {
                progressListener(totalBytes, receivedBytes, true);
              }
            },
          ),
        )
        .map((v) => utf8.decoder.convert(v))
        .join();

    return body;
  }

  static raw({
    String url,
    Map<String, dynamic> headers,
    HttpProgressListener downloadProgressListener,
    String method,
  }) async {
    return await getData(
      baseURL: url,
      endpoint: "",
      headers: headers,
      downloadProgressListener: downloadProgressListener,
      method: method,
      useAuthorization: false,
      returnAsJson: false,
    );
  }

  static getData({
    String baseURL,
    String endpoint,
    Map<String, dynamic> headers,
    HttpProgressListener downloadProgressListener,
    dynamic data,
    String method,
    bool useAuthorization = true,
    bool returnAsJson = true,
  }) async {
    final client = HttpClient();
    client.userAgent = "AIFramework/";
    client.connectionTimeout = Duration(seconds: 30);

    HttpClientRequest request;
    String mBaseURL = baseURL ?? Http.baseURL;

    var uri = Uri.parse("$mBaseURL$endpoint");

    if (method != null && method.isNotEmpty) {
      switch (method.toUpperCase()) {
        case "GET":
          request = await client.getUrl(uri);
          break;

        case "POST":
          request = await client.postUrl(uri);
          break;

        case "PUT":
          request = await client.putUrl(uri);
          break;

        case "DELETE":
          request = await client.deleteUrl(uri);
          break;

        case "PATCH":
          request = await client.patchUrl(uri);
          break;

        case "HEAD":
          request = await client.headUrl(uri);
          break;

        default:
          request = await client.getUrl(uri);
          break;
      }
    } else {
      if (data == null) {
        request = await client.getUrl(uri);
      } else {
        request = await client.postUrl(uri);
      }
    }

    _processHeader(
      request: request,
      headers: headers,
      useAuthorization: useAuthorization,
    );

    if (data != null) {
      String payload = json.encode(data);
      request.write(payload);
    }

    final response = await request.close();
    if (response.statusCode == 200) {
      if (response.contentLength > 0) {
        if (returnAsJson) {
          return json.decode(
            await _processResponse(
              response: response,
              progressListener: downloadProgressListener,
            ),
          );
        }
      }

      return await _processResponse(
        response: response,
        progressListener: downloadProgressListener,
      );
    }

    return null;
  }
}
